#!/bin/sh
# BrainF*ck compiler for SHARP PC-1500 (LH5801)
# Copyright 2015 Christophe Gottheimer <cgh75015@gmail.com>
#
# BrainF_ck compiler is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.  Note that I am not granting permission to
# redistribute or modify the BrainF_ck compiler under the terms of any later
# version of the General Public License.
# 
# This program is distributed in the hope that it will be useful (or at
# least amusing), but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program (in the file "COPYING"); if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


LHASMVERS=`lhasm -v# 2>/dev/null | awk 'BEGIN {FS="."}{ printf "%02u%02u%02u",$1,$2,$3 }'`
[ -z "$LHASMVERS" ] && \
        echo "The lhTools need to be installed." && \
        exit 1
[ $LHASMVERS -lt 000700 ] && \
        echo "BRAINF_CK requires The lhTools 0.7.0 minimum. Found lhTools `lhasm -v#`" && \
        exit 2
[ $LHASMVERS -ge 000702 ] && Z=yes 
[ $LHASMVERS -ge 000703 ] && P=yes 
[ $LHASMVERS -ge 000800 ] && G=yes 

rm -rf Images
mkdir -p Images

for f in 151 155 159 161
do
	GG=""
	ff=Images/bf-ce$f
	[ -n "$G" ] && GG="-G $ff.dbg"
	lhasm \
		-m pc1500 -m ce$f \
		$GG \
		-L $ff.lst \
		-F $ff.frg \
		-o $ff.bin brainf_ck.asm
	rc=$?
	[ $rc -ne 0 ] && echo "Aborted due assembler errors... " && exit $rc
	or=`grep ".FRAGMENTS:" $ff.frg | awk '{ print $2 }'`
	[ -z "$or" ] && \
		echo "No origin found! Fragments file $ff.frg seems corrupted. Abort." \
		&& exit 99
	bin2wav \
		-p 1500 \
		-t bin \
		-a 0x$or \
		-s 5 \
		-n 'BRAINF*CK'"-$f" \
		$ff.bin $ff.wav
	rc=$?
	[ $rc -ne 0 ] && echo "Aborted due bin2wav errors... " && exit $rc

	[ -n "$Z" ] && lhasm \
				-m pc1500 -m ce$f \
				-Zname="BRAINF*CK-$f" \
				-o $ff.bin158 brainf_ck.asm
	rc=$?
	[ -n "$Z" ] && [ $rc -ne 0 ] && echo "Aborted due assemble -Z errors... " && exit $rc
done

[ -n "$P" ] &&
	lhpoke \
		-A 'INPUT "BRAINF*CK BASE ?",A' \
		-L 100 -I 1 -B 16 -V A \
		-xx \
		-o brainf_ckbas.bas \
		$ff.bin
rc=$?
[ -n "$P" ] && [ $rc -ne 0 ] && echo "Aborted due binary->BASIC errors... " && exit $rc

for f in *.bas
do
	ff=`basename $f .bas`
	FF=`echo $ff | dd conv=ucase 2>/dev/null`
	cat $f | sed 's/\[/\(/g;s/\]/\)/g' | lhasm -i -o Images/$ff.bin
	rc=$?
	[ $rc -ne 0 ] && echo "Aborted due assembler errors... " && exit $rc

        nn
	bin2wav \
		-p 1500 \
		-t img \
		-s 5 \
		-n "$FF" \
		Images/$ff.bin Images/$ff.wav
	rc=$?
	[ $rc -ne 0 ] && echo "Aborted due bin2wav errors... " && exit $rc

	[ -n "$Z" ] && cat $f | \
		sed 's/\[/\(/g;s/\]/\)/g' | \
			lhasm -Zname=$FF -i -o Images/$ff.bin158
	rc=$?
	[ -n "$Z" ] && [ $rc -ne 0 ] && echo "Aborted due assemble -Z errors... " && exit $rc
done

exit 0
